import { EntityListener } from "../@interfaces/EntityListener";
import { ImmutableArray } from "../utils/ImmutableArray";
import { Component, ComponentClass } from "./Component";
import { Entity } from "./Entity";
import { EntitySystem } from "./EntitySystem";
import { Family } from "./Family";
import { FamilyManager } from "./FamilyManager";
export declare class Engine extends Map<Entity, Entity> {
    private static instance?;
    private static empty;
    private readonly componentAdded;
    private readonly componentRemoved;
    private systemManager;
    private entityManager;
    private familyManager;
    private updating;
    private notifying;
    /**
     * Приватный Конструтор, используйте {@link Engine.create()}
     */
    constructor();
    /**
     * Создает новый объект Entity
     * @return объект {@link Entity}
     */
    createEntity(): Entity;
    /**
     * Создает новый компонент {@link Component}. Чтобы использовать этот метод, ваши
     * компоненты должны иметь видимый конструктор без аргументов.
     * ```js
     * createComponent(MyComponent, arg1, arg2, ...args);
     * ```
     */
    createComponent<T extends Component>(componentClass: ComponentClass, ...args: any[]): T;
    /**
     * Добавляет сущность в этот движок.
     * Вызовет ошибку, если данный объект
     * уже был зарегистрирован в движке.
     */
    addEntity(entity: Entity): void;
    /**
     * Удаляет сущность из этого движка.
     */
    removeEntity(entity: Entity): void;
    removeAllEntities(): void;
    /**
     * Возвращает {@link ImmutableArray} из {@link Entity}, которым управляет Engine
     * но не может использоваться для изменения состояния. Этот массив не
     * является неизменным в том смысле, что его содержимое не будет изменено, а в том смысле, что оно
     * лишь отражает состояние движка.
     *
     * Массив является неизменяемым в том смысле, что вы не можете изменить его содержимое.
     * через API класса {@link ImmutableArray}, но вместо этого "управляется" движком
     * Двигатель может добавлять или удалять элементы из массива, и это будет отражено в
     * этом массиве.
     *
     * Это важное примечание, если вы перебираете возвращаемые объекты в цикле.
     * и вызов операций
     * который может добавлять/удалять сущности из движка, как базовый итератор
     * этот массив будут отражать эти изменения.
     *
     * В возвращаемом массиве будут удалены сущности, если они будут удалены в движке,
     * но нет возможности вводить новые сущности через интерфейс массива,
     * или удалить  сущности из движка через интерфейс этого массива.
     *
     *
     * @return Неизменяемый массив сущностей, которые будут соответствовать
     * состоянию сущности в движке.
     */
    getEntities(): ImmutableArray<Entity>;
    /**
     * Добавляет {@link EntitySystem} в этот движок.
     * Если в Движке уже была система того же класса,
     * новый заменит старый.
     */
    addSystem(system: EntitySystem): void;
    /**
     * Удаляет {@link EntitySystem} из этого Engine.
     */
    removeSystem(system: EntitySystem): void;
    /**
     * Удаляет все системы из этого двигателя.
     */
    removeAllSystems(): void;
    /**
     * Быстрый поиск {@link EntitySystem}.
     */
    getSystem<T extends EntitySystem>(systemType: any): T;
    /**
     * @return неизменяемый массив всех систем сущностей, управляемых {@link Engine}.
     */
    getSystems(): ImmutableArray<EntitySystem>;
    /**
     * Возвращает неизменяемую коллекцию сущностей для указанного {@link Family}.
     * Возвращает один и тот же экземпляр каждый раз для одного и того же семейства.
     */
    getEntitiesFor(family: Family): ImmutableArray<Entity>;
    addEntityListener(family: Family, priority: number, listener: EntityListener): void;
    removeEntityListener(listener: EntityListener): void;
    update(deltaTime: number): Promise<void>;
    addEntityInternal(entity: Entity): void;
    removeEntityInternal(entity: Entity): void;
    isUpdating(): boolean;
    isNotifying(): boolean;
    getFamilyManager(): FamilyManager;
    /**
     * Возвращает instance класса {@link Engine}.
     * Eсли экземпляр движка был создан ранее, возвращается этот экземпляр, в противном случае - создается новый.
     * @returns Engine
     */
    static create(): Engine;
}
